{ inputs, pkgs, lib, ... }:
{
  programs.yazi = {
    enable = true;
    enableFishIntegration = true;
    package = inputs.yazi.packages.${pkgs.system}.default;
    initLua = ./yazi/init.lua;

    plugins = with pkgs.yaziPlugins; {
      git = git;
      chmod = chmod;
      diff = diff;
      mount = mount;
      smart-paste = smart-paste;
      toggle-pane = toggle-pane;
      restore = restore;
      lazygit = lazygit;
      ouch = ouch;
      rich-preview = rich-preview;
      mediainfo= mediainfo;
      smart-filter = smart-filter;

      starship = pkgs.fetchFromGitHub {
        owner = "Rolv-Apneseth";
        repo = "starship.yazi";
        rev = "ee12c6b81f0debebf3c4395b00571e8cdbed6451";
        sha256 = "sha256-PYeR6fiWDbUMpJbTFSkM57FzmCbsB4W4IXXe25wLncg=";
      };

      open-with-cmd = pkgs.fetchFromGitHub {
        owner = "Ape";
        repo = "open-with-cmd.yazi";
        rev = "HEAD";
        sha256 = "sha256-QazKfNEPFdkHwMrH4D+VMwj8fGXM8KHDdSvm1tik3dQ=";
      };

      office = pkgs.fetchFromGitHub {
        owner = "macydnah";
        repo = "office.yazi";
        rev = "HEAD";
        sha256 = "sha256-XE+EfVPsO09zG8qYEhN6O95mS9NJlTdOd4Gsem2KtPI=";
      };

      exifaudio = pkgs.fetchFromGitHub {
        owner = "Sonico98";
        repo = "exifaudio.yazi";
        rev = "HEAD";
        sha256 = "sha256-aniuY14pXcoaW6YkUwt7hTl9mWjl5HoOPhHkuY4ooAw=";
      };

      time-travel = pkgs.fetchFromGitHub {
        owner = "iynaix";
        repo = "time-travel.yazi";
        rev = "HEAD";
        sha256 = "sha256-ZZgn5rsBzvZcnDWZfjMBPRg9QUz4FTq5UIPWfnwXHQs=";
      };

      pref-by-location = pkgs.fetchFromGitHub {
        owner = "boydaihungst";
        repo = "pref-by-location.yazi";
        rev = "HEAD";
        sha256 = "sha256-miFoKtMKbN58wH+f+auk2YLVfTTwyvuTW1VAHrNL5fA=";
      };

      # dir-rules = ./yazi/plugins/dir-rules.yazi;
      smart-tab = ./yazi/plugins/smart-tab.yazi;
    };

    keymap = {
      mgr.prepend_keymap = [
        { on   = "!"; run  = "shell '$SHELL' --block"; desc = "Open shell here"; }

        { on = ["c" "d"]; run = "shell 'ripdrag \"$@\" -x 2>/dev/null &' --confirm"; desc = "Drag selection";}
        { on = ["c" "c"]; run = ["shell 'for path in \"$@\"; do echo \"file://$path\"; done | wl-copy -t text/uri-list'\n" "yank"]; desc = "Copy file (yank)"; }
        { on = ["c" "p"]; run = "copy path"; desc = "Copy file path"; }
        { on = ["c" "u"]; run = "unyank"; desc = "Cancel file copy"; }

        { on = ["g" "y"]; run = "cd ~/sync"; desc = "Go to ~/sync"; }
        { on = ["g" "p"]; run = "cd ~/projects"; desc = "Go to ~/projects"; }
        { on = ["g" "s"]; run = "cd ~/source"; desc = "Go to ~/source"; }
        { on = ["g" "d"]; run = "cd ~/downloads"; desc = "Go to ~/downloads"; }


        { on = "p"; run = "plugin smart-paste"; }
        { on = "P"; run = "plugin smart-paste --force"; }

        { on = ["t" "p"]; run  = "plugin toggle-pane min-preview";desc = "Show or hide the preview pane"; }
        { on = ["t" "P"]; run  = "plugin toggle-pane max-preview";desc = "Maximize/restore the preview pane"; }
        { on = ["t" "g"]; run  = "plugin lazygit"; desc = "Lazygit"; }

        # { on = ["b" "p"]; run  = "plugin time-travel --args=prev"; desc = "Previous snapshot"; }
        # { on = ["b" "n"]; run  = "plugin time-travel --args=next"; desc = "Next snapshot"; }
        # { on = ["b" "q"]; run  = "plugin time-travel --args=exit"; desc = "Exit snapshot"; }

        { on = "<C-t>"; run ="tab_create"; }
        { on = "<C-w>"; run ="close"; }
        { on = "<C-A-right>"; run ="tab_switch 1 --relative"; }
        { on = "<C-A-left>"; run ="tab_switch -1 --relative"; }
        { on = "<C-A-h>"; run ="tab_switch 1 --relative"; }
        { on = "<C-A-l>"; run ="tab_switch -1 --relative"; }
        { on = "<C-1>"; run ="tab_switch 0"; }
        { on = "<C-2>"; run ="tab_switch 1"; }
        { on = "<C-3>"; run ="tab_switch 2"; }
        { on = "<C-4>"; run ="tab_switch 3"; }
        
        { on = "<Tab>"; run = "spot"; desc = "Spot hovered file"; }

        { on = "M"; run = "plugin mount"; }

        { on = "C"; run = "plugin ouch zip"; }

        { on = "T"; run = "plugin smart-tab"; desc = "Create a tab and enter the hovered directory"; }

        { on = "<Backspace>"; run = "remove"; }
        { on = ["d" "d"]; run = "remove"; desc = "Delete files/folders"; }
        { on = ["d" "u"]; run = "plugin restore"; desc = "Restore last deleted files/folders"; }
        { on = ["d" "x"]; run = "shell 'trash-empty'"; desc = "Empty trash"; }

        { on = "F"; run = "filter --smart"; desc = "Find file"; }
        { on = "f"; run = "find --smart"; desc = "Find file"; }

        { on = "z"; run = "plugin zoxide"; }
        { on = "Z"; run = "plugin fzf"; }

        { on = "."; run = [ "hidden toggle" "plugin pref-by-location -- save" ]; desc = "Toggle the visibility of hidden files";}

        { on = [ "m" "d" ]; run = [ "linemode time_and_size" "plugin pref-by-location -- save" ]; desc = "Linemode: time & size (default)"; }
        { on = [ "m" "s" ]; run = [ "linemode size" "plugin pref-by-location -- save" ]; desc = "Linemode: size"; }
        { on = [ "m" "p" ]; run = [ "linemode permissions" "plugin pref-by-location -- save" ]; desc = "Linemode: permissions"; }
        { on = [ "m" "b" ]; run = [ "linemode btime" "plugin pref-by-location -- save" ]; desc = "Linemode: btime"; }
        { on = [ "m" "m" ]; run = [ "linemode mtime" "plugin pref-by-location -- save" ]; desc = "Linemode: mtime"; }
        { on = [ "m" "o" ]; run = [ "linemode owner" "plugin pref-by-location -- save" ]; desc = "Linemode: owner"; }
        { on = [ "m" "n" ]; run = [ "linemode none" "plugin pref-by-location -- save" ]; desc = "Linemode: none"; }

        { on = [ "," "t" ]; run = "plugin pref-by-location -- toggle"; desc = "Toggle auto-save preferences"; }
        { on = [ "," "d" ]; run = "plugin pref-by-location -- disable"; desc = "Disable auto-save preferences"; }

        # This will reset any preference changes for the current working directory (CWD)
        # then fall back to the predefined preferences in init.lua or yazi.toml
        { on = [ "," "R" ]; run = [ "plugin pref-by-location -- reset" ]; desc = "Reset preference of cwd"; }
        { on = [ "," "m" ]; run = [ "sort mtime --reverse=no" "linemode mtime" "plugin pref-by-location -- save" ]; desc = "Sort by modified time"; }
        { on = [ "," "M" ]; run = [ "sort mtime --reverse" "linemode mtime" "plugin pref-by-location -- save" ]; desc = "Sort by modified time (reverse)"; }
        { on = [ "," "b" ]; run = [ "sort btime --reverse=no" "linemode btime" "plugin pref-by-location -- save" ]; desc = "Sort by birth time"; }
        { on = [ "," "B" ]; run = [ "sort btime --reverse" "linemode btime" "plugin pref-by-location -- save" ]; desc = "Sort by birth time (reverse)"; }
        { on = [ "," "e" ]; run = [ "sort extension --reverse=no" "plugin pref-by-location -- save" ]; desc = "Sort by extension"; }
        { on = [ "," "E" ]; run = [ "sort extension --reverse" "plugin pref-by-location -- save" ]; desc = "Sort by extension (reverse)"; }
        { on = [ "," "a" ]; run = [ "sort alphabetical --reverse=no" "plugin pref-by-location -- save" ]; desc = "Sort alphabetically"; }
        { on = [ "," "A" ]; run = [ "sort alphabetical --reverse" "plugin pref-by-location -- save" ]; desc = "Sort alphabetically (reverse)"; }
        { on = [ "," "n" ]; run = [ "sort natural --reverse=no" "plugin pref-by-location -- save" ]; desc = "Sort naturally"; }
        { on = [ "," "N" ]; run = [ "sort natural --reverse" "plugin pref-by-location -- save" ]; desc = "Sort naturally (reverse)"; }
        { on = [ "," "s" ]; run = [ "sort size --reverse=no" "linemode size" "plugin pref-by-location -- save" ]; desc = "Sort by size"; }
        { on = [ "," "S" ]; run = [ "sort reverse" "linemode size" "plugin pref-by-location -- save" ]; desc = "Sort by size (reverse)"; }
        { on = [ "," "r" ]; run = [ "sort random --reverse=no" "plugin pref-by-location -- save" ]; desc = "Sort randomly"; }

      ];
    };
    settings = {
      mgr = {
        sort_dir_first = false;
        linemode = "time_and_size";
        show_symlink = true;
      };
      opener = {
        edit = [
          { run = "emacsclient -n -r \"$@\""; desc = "emacsclient"; orphan = true; block = false; }
          { run = "nano \"$@\""; desc = "nano"; orphan = false; block = true; }
        ];
        extract = [
          { run = "ouch d -y \"$@\""; desc = "Extract here with ouch"; }
        ];
      };
      open = {
        prepend_rules = [
          { name = "*.zip"; use = "extract"; }
        ];
      };
      plugin = {
        prepend_preloaders = [
          { name = "/home/emenel/mounts/**"; run = "noop"; }
          { name = "/mnt/**"; run = "noop"; }

          # Office Documents
          { mime = "application/openxmlformats-officedocument.*"; run = "office"; }
          { mime = "application/oasis.opendocument.*"; run = "office"; }
          { mime = "application/ms-*"; run = "office"; }
          { mime = "application/msword"; run = "office"; }
          { name = "*.docx"; run = "office"; }
        ];

        prepend_previewers = [
          { mime = "audio/*"; run = "exifaudio"; }

          { name = "*.csv"; run = "rich-preview"; }
          { name = "*.md"; run = "rich-preview" ; }
          { name = "*.rst"; run = "rich-preview"; }
          { name = "*.ipynb"; run = "rich-preview"; }
          { name = "*.json"; run = "rich-preview"; }
          { name = "*.org"; run = "rich-preview"; }

          # Office Documents
          { mime = "application/openxmlformats-officedocument.*"; run = "office"; }
          { mime = "application/oasis.opendocument.*"; run = "office"; }
          { mime = "application/ms-*"; run = "office"; }
          { mime = "application/msword"; run = "office"; }
          { name = "*.docx"; run = "office"; }
        ];

        append_previewers = [
          # Archive previewer
          { name = "*.zip"; run = "ouch"; }
          { mime = "application/*zip";            run = "ouch"; }
          { mime = "application/x-tar";           run = "ouch"; }
          { mime = "application/x-bzip2";         run = "ouch"; }
          { mime = "application/x-7z-compressed"; run = "ouch"; }
          { mime = "application/x-rar";           run = "ouch"; }
          { mime = "application/x-xz";            run = "ouch"; }
          { mime = "application/octet-stream";    run = "ouch"; }
        ];

        _spotters = [
          { name = "*"; run = "file-extra-metadata"; }
        ];
        prepend_fetchers = [
          # {
          #   id   = "git";
          #   name = "*";
          #   run  = "git";
          # }
          # {
          #   id   = "git";
          #   name = "*/";
          #   run  = "git";
          # }
        ];
      };
      tasks = {
        image_alloc = 53687091;
        image_bound = [ 0 0 ];
      };
    };
  };

  xdg.configFile."yazi/theme.toml".source = ./yazi/theme.toml;
  xdg.configFile."yazi/flavors".source = ./yazi/flavors;
}
